@@ -9,4 +9,4 @@ pub use quadratic::QuadExtension;
 mod cubic;
 pub use cubic::CubeExtension;
 
-use super::{ExtensibleField, FieldElement};
+use super::{ExtensibleField, ExtensionOf, FieldElement};
