@@ -9,10 +9,7 @@
 //! fast modular arithmetic including branchless multiplication and addition. Base elements are
 //! stored in the Montgomery form using `u64` as the backing type.
 
-use super::{
-    traits::{FieldElement, StarkField},
-    ExtensibleField,
-};
+use super::{ExtensibleField, FieldElement, StarkField};
 use core::{
     convert::{TryFrom, TryInto},
     fmt::{Debug, Display, Formatter},
@@ -299,6 +296,11 @@ impl ExtensibleField<2> for BaseElement {
         [z + a[1] * b[1], (a[0] + a[1]) * (b[0] + b[1]) - z]
     }
 
+    #[inline(always)]
+    fn mul_base(a: [Self; 2], b: Self) -> [Self; 2] {
+        [a[0] * b, a[1] * b]
+    }
+
     #[inline(always)]
     fn frobenius(x: [Self; 2]) -> [Self; 2] {
         [x[0] + x[1], -x[1]]
@@ -339,6 +341,11 @@ impl ExtensibleField<3> for BaseElement {
         ]
     }
 
+    #[inline(always)]
+    fn mul_base(a: [Self; 3], b: Self) -> [Self; 3] {
+        [a[0] * b, a[1] * b, a[2] * b]
+    }
+
     #[inline(always)]
     fn frobenius(x: [Self; 3]) -> [Self; 3] {
         // coefficients were computed using SageMath
