@@ -14,10 +14,7 @@
 //!
 //! Internally, the values are stored in the range $[0, 2^{64})$ using `u64` as the backing type.
 
-use super::{
-    traits::{FieldElement, StarkField},
-    ExtensibleField,
-};
+use super::{ExtensibleField, FieldElement, StarkField};
 use core::{
     convert::{TryFrom, TryInto},
     fmt::{Debug, Display, Formatter},
@@ -99,51 +96,33 @@ impl FieldElement for BaseElement {
     #[inline]
     #[allow(clippy::many_single_char_names)]
     fn inv(self) -> Self {
-        let x = self.as_int();
+        // compute base^(M - 2) using 72 multiplications
+        // M - 2 = 0b1111111111111111111111111111111011111111111111111111111111111111
 
-        if x == 0 {
-            return Self::ZERO;
-        };
+        // compute base^11
+        let t2 = self.square() * self;
 
-        let mut a: u128 = 0;
-        let mut u: u128 = if x & 1 == 1 {
-            x as u128
-        } else {
-            (x as u128) + (M as u128)
-        };
-        let mut v: u128 = M as u128;
-        let mut d = (M as u128) - 1;
+        // compute base^111
+        let t3 = t2.square() * self;
 
-        while v != 1 {
-            while v < u {
-                u -= v;
-                d += a;
-                while u & 1 == 0 {
-                    if d & 1 == 1 {
-                        d += M as u128;
-                    }
-                    u >>= 1;
-                    d >>= 1;
-                }
-            }
+        // compute base^111111 (6 ones)
+        let t6 = exp_acc::<3>(t3, t3);
 
-            v -= u;
-            a += d;
+        // compute base^111111111111 (12 ones)
+        let t12 = exp_acc::<6>(t6, t6);
 
-            while v & 1 == 0 {
-                if a & 1 == 1 {
-                    a += M as u128;
-                }
-                v >>= 1;
-                a >>= 1;
-            }
-        }
+        // compute base^111111111111111111111111 (24 ones)
+        let t24 = exp_acc::<12>(t12, t12);
 
-        while a > (M as u128) {
-            a -= M as u128;
-        }
+        // compute base^1111111111111111111111111111111 (31 ones)
+        let t30 = exp_acc::<6>(t24, t6);
+        let t31 = t30.square() * self;
 
-        Self(a as u64)
+        // compute base^111111111111111111111111111111101111111111111111111111111111111
+        let t63 = exp_acc::<32>(t31, t31);
+
+        // compute base^1111111111111111111111111111111011111111111111111111111111111111
+        t63.square() * self
     }
 
     fn conjugate(&self) -> Self {
@@ -370,6 +349,13 @@ impl ExtensibleField<2> for BaseElement {
         ]
     }
 
+    #[inline(always)]
+    fn mul_base(a: [Self; 2], b: Self) -> [Self; 2] {
+        // multiplying an extension field element by a base field element requires just 2
+        // multiplications in the base field.
+        [a[0] * b, a[1] * b]
+    }
+
     #[inline(always)]
     fn frobenius(x: [Self; 2]) -> [Self; 2] {
         [x[0] + x[1], -x[1]]
@@ -410,6 +396,13 @@ impl ExtensibleField<3> for BaseElement {
         ]
     }
 
+    #[inline(always)]
+    fn mul_base(a: [Self; 3], b: Self) -> [Self; 3] {
+        // multiplying an extension field element by a base field element requires just 3
+        // multiplications in the base field.
+        [a[0] * b, a[1] * b, a[2] * b]
+    }
+
     #[inline(always)]
     fn frobenius(x: [Self; 3]) -> [Self; 3] {
         // coefficients were computed using SageMath
@@ -569,3 +562,13 @@ fn mod_reduce(x: u128) -> u64 {
     let (result, over) = tmp0.overflowing_add(tmp1);
     result.wrapping_add(E * (over as u64))
 }
+
+/// Squares the base N number of times and multiplies the result by the tail value.
+#[inline(always)]
+fn exp_acc<const N: usize>(base: BaseElement, tail: BaseElement) -> BaseElement {
+    let mut result = base;
+    for _ in 0..N {
+        result = result.square();
+    }
+    result * tail
+}
